@@ -1,11 +1,12 @@
-#!/usr/bin/perl
 # ----------------------------------------------------------------
     package XML::TreePP;
     use strict;
-    use Symbol;
     use Carp;
+    use Symbol;
+#   use warnings;
+# ----------------------------------------------------------------
     use vars qw( $VERSION );
-    $VERSION = "0.02";
+    $VERSION = "0.03";
 # ----------------------------------------------------------------
     my $XML_ENCODING = "UTF-8";
     my $INTERNAL_ENCODING = "UTF-8";
@@ -17,7 +18,7 @@ XML::TreePP -- Pure Perl implementation for parsing/writing xml file
 
 =head1 SYNOPSIS
 
-# parse xml file into hash tree
+parse xml file into hash tree
 
     use XML::TreePP;
     my $tpp = XML::TreePP->new();
@@ -25,7 +26,7 @@ XML::TreePP -- Pure Perl implementation for parsing/writing xml file
     print "Title: ", $tree->{"rdf:RDF"}->{item}->[0]->{title}, "\n";
     print "URL:   ", $tree->{"rdf:RDF"}->{item}->[0]->{link}, "\n";
 
-# write xml from hash tree
+write xml as string from hash tree
 
     use XML::TreePP;
     my $tpp = XML::TreePP->new();
@@ -39,7 +40,7 @@ XML::TreePP -- Pure Perl implementation for parsing/writing xml file
     my $xml = $tpp->write( $tree );
     print $xml;
 
-# get remote xml file with HTTP-GET and parse it into hash tree
+get remote xml file with HTTP-GET and parse it into hash tree
 
     use XML::TreePP;
     my $tpp = XML::TreePP->new();
@@ -47,7 +48,7 @@ XML::TreePP -- Pure Perl implementation for parsing/writing xml file
     print "Title: ", $tree->{"rdf:RDF"}->{channel}->{title}, "\n";
     print "URL:   ", $tree->{"rdf:RDF"}->{channel}->{link}, "\n";
 
-# get remote xml file with HTTP-POST and parse it into hash tree
+get remote xml file with HTTP-POST and parse it into hash tree
 
     use XML::TreePP;
     my $tpp = XML::TreePP->new( array_element => [qw( item )] );
@@ -68,6 +69,8 @@ like XMLHttpRequest object at JavaScript language.
 
 =head1 EXAMPLES
 
+=head2 Parse XML file
+
 Sample XML source:
 
     <?xml version="1.0" encoding="UTF-8"?>
@@ -107,22 +110,24 @@ Result dumped:
         }
     };
 
-Note:
+Explain:
 
-    The father's given name is in $tree->{family}->{father}.
-    The family name of the family is in $tree->{family}->{"-name"}.
-    The prefix '-' is added on every attributes' name.
+    print $tree->{family}->{father};        # the father's given name.
 
-    The second boy's name is in $tree->{family}->{children}->{boy}->[1].
-    The array is used because the family has two boys.
+The prefix '-' is added on every attributes' name.
 
-    The girl's name is in $tree->{family}->{children}->{girl}.
+    print $tree->{family}->{"-name"};       # the family name of the family
 
-Text node and attributes:
+The array is used because the family has two boys.
 
-    If a element has both of a text node and attributes
-    or both of a text node and other child nodes,
-    value of a text node is moved to "#text" like child nodes.
+    print $tree->{family}->{children}->{boy}->[1];  # The second boy's name
+    print $tree->{family}->{children}->{girl};      # The girl's name
+
+=head2 Text node and attributes:
+
+If a element has both of a text node and attributes
+or both of a text node and other child nodes,
+value of a text node is moved to "#text" like child nodes.
 
     use XML::TreePP;
     use Data::Dumper;
@@ -141,56 +146,76 @@ The result dumped is following:
         }
     };
 
-    The special node name of "#text" is used because this elements 
-    has attribute(s) in addition to the text node.
+The special node name of "#text" is used because this elements 
+has attribute(s) in addition to the text node.
 
 =head1 METHODS
 
-    my $tpp = XML::TreePP->new( %attributes )
-        This method constructs a new "XML::TreePP" object.
-        Optional attributes are following:
+=head2 $tpp = XML::TreePP->new( %attributes )
 
-    output_encoding => "UTF-8",
-        You can define a encoding of xml file generated by write/writefile 
-        methods. On Perl 5.8.x and later, you can select it from every 
-        encodings supported by Encode.pm. On Perl 5.6.x or before with 
-        Jcode.pm, you can use "Shift_JIS", "EUC-JP", "ISO-2022-JP" and 
-        "UTF-8". The default value is "UTF-8".
+This method constructs a new "XML::TreePP" object.
+Optional attributes are following:
 
-    array_element => [],
-        This option allows you to specify a list of element names which 
-        should always be forced into an array representation
-        The default value is null, it means that context of the elements 
-        will determine to make array or to keep it scalar.
+=head3 output_encoding => "UTF-8",
 
-    my $tree = $tpp->parse( $source );
-        This method reads XML source and returns a hash tree converted.
-        The first argument is a scalar or a reference to a scalar.
+You can define a encoding of xml file generated by write/writefile 
+methods. On Perl 5.8.x and later, you can select it from every 
+encodings supported by Encode.pm. On Perl 5.6.x or before with 
+Jcode.pm, you can use "Shift_JIS", "EUC-JP", "ISO-2022-JP" and 
+"UTF-8". The default value is "UTF-8".
 
-    my $tree = $tpp->parsefile( $file );
-        This method reads a XML file and returns a hash tree converted.
-        The first argument is a filename.
+=head3 array_element => [ "rdf:li", "item", "-xmlns" ],
 
-    my $tree = $tpp->parsefile( $method, $url, $body );
-        This method receive a XML file from a remote server via HTTP and 
-        returns a hash tree converted. 
-        This medhot requires LWP::UserAgent module.
-        $method is a method of HTTP connection: GET/POST/PUT/DELETE
-        $url is URI of a XML file. 
-        $body is request body when you use POST method.
+This option allows you to specify a list of element names which 
+should always be forced into an array representation
+The default value is null, it means that context of the elements 
+will determine to make array or to keep it scalar.
 
-    my $source = $tpp->write( $tree );
-        This method parse a hash tree and returns a XML source generated.
-        $tree is a referecen to a hash tree.
+=head3 first_out => [ "link", "title", "-type" ],
 
-    $tpp->writefile( $file, $tree );
-        This method parse a hash tree and writes a XML source into a file.
-        $file is a filename to create.
-        $tree is a referecen to a hash tree.
+This option allows you to specify a list of element/attribute 
+names which should always appears at first on output XML code.
+The default value is null, it means alphabetical order is used.
+
+=head3 last_out => [ "items", "item", "entry" ],
+
+This option allows you to specify a list of element/attribute 
+names which should always appears at last on output XML code.
+
+=head2 $tree = $tpp->parse( $source );
+
+This method reads XML source and returns a hash tree converted.
+The first argument is a scalar or a reference to a scalar.
+
+=head2 $tree = $tpp->parsefile( $file );
+
+This method reads a XML file and returns a hash tree converted.
+The first argument is a filename.
+
+=head2 $tree = $tpp->parsefile( $method, $url, $body );
+
+This method receive a XML file from a remote server via HTTP and 
+returns a hash tree converted. 
+This medhot requires LWP::UserAgent module.
+$method is a method of HTTP connection: GET/POST/PUT/DELETE
+$url is URI of a XML file. 
+$body is request body when you use POST method.
+
+=head2 $source = $tpp->write( $tree, $encode );
+
+This method parse a hash tree and returns a XML source generated.
+$tree is a referecen to a hash tree.
+
+=head2 $tpp->writefile( $file, $tree, $encode );
+
+This method parse a hash tree and writes a XML source into a file.
+$file is a filename to create.
+$tree is a referecen to a hash tree.
 
 =head1 AUTHOR
 
 Yusuke Kawasaki, E<lt>u-suke [at] kawa.netE<gt>
+http://www.kawa.net/works/perl/treepp/treepp-e.html
 
 =head1 COPYRIGHT AND LICENSE
 
@@ -220,12 +245,22 @@ sub write {
     my $self = shift;
     my $tree = shift;
     my $from = $self->{internal_encoding} || $INTERNAL_ENCODING;
-    my $to = $self->{output_encoding} || $XML_ENCODING;
+    my $to = shift || $self->{output_encoding} || $XML_ENCODING;
     my $decl = '<?xml version="1.0" encoding="'.$to.'" ?>';
-    my $text = &hash_to_xml( undef, $tree );
+    if ( exists $self->{first_out} ) {
+        my $keys = $self->{first_out};
+        $keys = [ $keys ] unless ref $keys;
+        $self->{__first_out} = { map {$_=>1} @$keys };
+    }
+    if ( exists $self->{last_out} ) {
+        my $keys = $self->{last_out};
+        $keys = [ $keys ] unless ref $keys;
+        $self->{__last_out} = { map {$_=>1} @$keys };
+    }
+    my $text = $self->hash_to_xml( undef, $tree );
     if ( $from && $to ) {
         my $stat = &encode_from_to( \$text, $from, $to );
-        croak "Unsupported encoding: $to\n" unless $stat;
+        Carp::croak "Unsupported encoding: $to" unless $stat;
     }
     join( "\n", $decl, $text );
 }
@@ -236,10 +271,11 @@ sub parsehttp {
     my $url = shift;
     my $body = shift;
 
+    local $@;
     eval { require LWP::UserAgent; } unless defined $LWP::UserAgent::VERSION;
-    croak "LWP::UserAgent is required: $@\n" if $@;
+    Carp::croak "LWP::UserAgent is required: $url" if $@;
     eval { require HTTP::Request; } unless defined $HTTP::Request::VERSION;
-    croak "HTTP::Request is required: $@\n" if $@;
+    Carp::croak "HTTP::Request is required: $url" if $@;
 
     my $ua = LWP::UserAgent->new();
     $ua->timeout(10);
@@ -270,7 +306,7 @@ sub parse {
         my $from = &xml_decl_encoding( $textref );
         if ( $from ) {
             my $stat = &encode_from_to( $textref, $from, $to );
-            croak "Unsupported encoding: $from\n" unless $stat;
+            Carp::croak "Unsupported encoding: $from" unless $stat;
         }
     }
     if ( exists $self->{force_array} ) {
@@ -368,7 +404,7 @@ sub flat_to_tree {
         my $name = $node->{tagName};
         if ( $node->{endTag} ) {
             last if ( $parent eq $name );
-            croak "Invalid tag sequence: <$parent></$name>\n";
+            Carp::croak "Invalid tag sequence: <$parent></$name>";
         }
         my $elem = $node->{attributes};
         if ( $node->{startTag} ) {
@@ -411,31 +447,40 @@ sub flat_to_tree {
 }
 # ----------------------------------------------------------------
 sub hash_to_xml {
+    my $self = shift;
     my $name = shift;
     my $hash = shift;
     my $out = [];
     my $attr = [];
-    foreach my $key ( sort grep { /^-/} keys %$hash ) {
-        my $val = &xml_unescape($hash->{$key});
-        $key =~ s/^-//;
-        push( @$attr, " ".$key.'="'.$val.'"' );
-    }
-    my $jattr = join( " ", @$attr );
-    foreach my $key ( sort grep {!/^-/} keys %$hash ) {
-        my $val = $hash->{$key};
-        if ( ref $val eq "ARRAY" ) {
-            my $child = &array_to_xml( $key, $val );
-            push( @$out, $child );
-        } elsif ( ref $val ) {
-            my $child = &hash_to_xml( $key, $val );
-            push( @$out, $child );
-        } elsif ( $val ne "" ) {
-            my $child = &scalar_to_xml( $key, $val );
-            push( @$out, $child );
-        } else {
-            push( @$out, "<$key />" );
+    my $allkeys = [ sort keys %$hash ];
+    my $firstkeys = [ grep {$self->{__first_out}->{$_}} @$allkeys ] if ref $self->{__first_out};
+    my $lastkeys  = [ grep {$self->{__last_out}->{$_}}  @$allkeys ] if ref $self->{__last_out};
+    $allkeys = [ grep {! $self->{__first_out}->{$_}} @$allkeys ] if ref $self->{__first_out};
+    $allkeys = [ grep {! $self->{__last_out}->{$_}}  @$allkeys ] if ref $self->{__last_out};
+    foreach my $loopkey ( $firstkeys, $allkeys, $lastkeys ) {
+        next unless ref $loopkey;
+        foreach my $key ( grep {!/^-/} @$loopkey ) {
+            my $val = $hash->{$key};
+            if ( UNIVERSAL::isa( $val, "ARRAY" ) ) {
+                my $child = $self->array_to_xml( $key, $val );
+                push( @$out, $child );
+            } elsif ( ref $val ) {
+                my $child = $self->hash_to_xml( $key, $val );
+                push( @$out, $child );
+            } elsif ( $val ne "" ) {
+                my $child = $self->scalar_to_xml( $key, $val );
+                push( @$out, $child );
+            } else {
+                push( @$out, "<$key />" );
+            }
+        }
+        foreach my $key ( grep { /^-/} @$loopkey ) {
+            my $val = &xml_unescape($hash->{$key});
+            $key =~ s/^-//;
+            push( @$attr, " ".$key.'="'.$val.'"' );
         }
     }
+    my $jattr = join( "", @$attr );
 #   s/^(\s*<)/  $1/mg foreach @$out;
     my $text = join( "", @$out );
     if ( defined $name ) {
@@ -453,18 +498,19 @@ sub hash_to_xml {
 }
 # ----------------------------------------------------------------
 sub array_to_xml {
+    my $self = shift;
     my $name = shift;
     my $array = shift;
     my $out = [];
     foreach my $val ( @$array ) {
-        if ( ref $val eq "ARRAY" ) {
-            my $child = &array_to_xml( $name, $val );
+        if ( UNIVERSAL::isa( $val, "ARRAY" ) ) {
+            my $child = $self->array_to_xml( $name, $val );
             push( @$out, $child );
         } elsif ( ref $val ) {
-            my $child = &hash_to_xml( $name, $val );
+            my $child = $self->hash_to_xml( $name, $val );
             push( @$out, $child );
         } elsif ( $val ne "" ) {
-            my $child = &scalar_to_xml( $name, $val );
+            my $child = $self->scalar_to_xml( $name, $val );
             push( @$out, $child );
         } else {
             push( @$out, "<$name />\n" );
@@ -476,6 +522,7 @@ sub array_to_xml {
 }
 # ----------------------------------------------------------------
 sub scalar_to_xml {
+    my $self = shift;
     my $name = shift;
     my $scalar = shift;
     my $copy = $scalar;
@@ -487,7 +534,7 @@ sub scalar_to_xml {
 sub write_raw_xml {
     my $file = shift;
     my $fh = Symbol::gensym();
-    open( $fh, ">$file" ) or croak "$! - $file\n";
+    open( $fh, ">$file" ) or Carp::croak "$! - $file";
     print $fh @_;
     close( $fh );
 }
@@ -495,7 +542,7 @@ sub write_raw_xml {
 sub read_raw_xml {
     my $file = shift;
     my $fh = Symbol::gensym();
-    open( $fh, $file ) or croak "$! - $file\n";
+    open( $fh, $file ) or Carp::croak "$! - $file";
     local $/ = undef;
     my $text = <$fh>;
     close( $fh );
@@ -521,7 +568,7 @@ sub encode_from_to {
         Encode::from_to( $$str, $from, $to, Encode::FB_XMLCREF() );
     } elsif (( uc($from) eq "ISO-8859-1" || uc($from) eq "LATIN-1" ) &&
                uc($to) eq "UTF-8" ) {
-		&latin1_to_utf8( $str );
+        &latin1_to_utf8( $str );
     } else {
         my $jfrom = &get_jcode_name( $from );
         my $jto   = &get_jcode_name( $to );
@@ -531,10 +578,10 @@ sub encode_from_to {
             if ( defined $Jcode::VERSION ) {
                 Jcode::convert( $str, $jto, $jfrom );
             } else {
-                croak "Encode.pm or Jcode.pm is required: $from to $to\n";
+                Carp::croak "Encode.pm or Jcode.pm is required: $from to $to";
             }
         } else {
-            croak "Encode.pm is required: $from to $to\n";
+            Carp::croak "Encode.pm is required: $from to $to";
         }
     }
     $to;
